/* IBM Confidential
* OCO Source Materials
* 5724-U98, 5655-V48, 5724-U50, 5655-V46
*  Copyright IBM Corp. 2013
* The source code for this program is not published or other -
* wise divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
*/
define(["dojo/_base/declare",
        "dojo/_base/lang",
        "dijit/_Widget",        
        "dijit/_TemplatedMixin",
        "dijit/_WidgetsInTemplateMixin",
        "dojo/text!./templates/UserRegistry.html",
        "dojo/i18n!../nls/user_resources",
        "dojo/dom-construct",
        "dojo/data/ItemFileWriteStore", 
        "dojox/grid/EnhancedGrid",
        "admin/users/dialogs/ImportUserWizardDialog",
        "dojo/request/xhr",
        "../dialogs/BusyDialog",
        "../dialogs/EditEmailDialog",
        "dojo/dom-style",
        "dijit/form/Button",
        "dojo/topic",
        "dijit/Toolbar",
        "dijit/form/TextBox",
        "dijit/layout/StackContainer",
        "dijit/layout/BorderContainer",
        "dijit/layout/ContentPane"
        ],
		function(dDeclare,
				dLang,
				dWidget,
				dTemplatedMixin,
				dWidgetsInTemplateMixin,
				templateText,
				resources,
				domConstruct,
				ItemFileWriteStore,
				EnhancedGrid,
				ImportUserWizardDialog,
				xhr,
				BusyDialog,
				EditEmailDialog,
				domStyle,
				Button,
				topic) {
return dDeclare("admin.users.views.UserRegistry", [dWidget,dTemplatedMixin,dWidgetsInTemplateMixin], 
{
	/**
	 * @private
	 */
	baseClass: "UserRegistry",
	
	/**
	 * The path to the widget template for the dijit._TemplatedMixin base class.
	 * 
	 * @private
	 * @constant
	 * @type String
	 */
	templateString: templateText,
	
	/** 
	 * Allow users to add other components (e.g. hidden fields) under password field
	 * 
	 * @private
	 */
	isContainer: false,
	/**
	 * The description string
	 */
	descriptionText: resources["userRegistryDescription"],
	/**
	 * The import label
	 */
	importLabel: resources["importUserButtonLabel"],
	/**
	 * The refresh label
	 */
	refreshLabel: resources["refreshButtonLabel"],
	/**
	 * The synchronize label
	 */
	synchronizeLabel: resources["synchronizeLDAPButtonLabel"],
	
	/**
	 * The synchronize with LDAP message
	 */
	synchonizeMessage: resources["synchronizingWithLDAPMessage"],
	/**
	 * The synchronized with LDAP failed message
	 */
	synchonizeFailedMessage: resources["synchronizingWithLDAPFailedMessage"],
	/**
	 * Edit email label
	 */
	editEmailLabel: resources["editEmail"],
	/**
	 * The user grid
	 */
	userGrid: null,
	/**
	 * The ldap server info
	 */
	ldapInfo: null,
	/**
	 * The busy dialog handle
	 */
	busyDialog: null,
	/**
	 * The synchronize button handle
	 */
	synchronizeButton: null,
	/**
	 * The edit email button handle
	 */
	editEmailButton: null,
	
	/**
	 * The startup function
	 */
	startup: function(){
		  this.inherited(arguments);
		  this.importButton.on("click", dLang.hitch(this, "importUsers"));
		  this.refreshButton.on("click", dLang.hitch(this, "retrieveUsers"));
		  var layout = [[
		                 {'name':'Logon ID', 'field':'userId', width:'15%'},
		                 {'name':'Name', 'field':'userName', width:"30%"},
		                 {'name':'Email', 'field':'userEmail', width: "55%"},
		                 ]];
		  this.userGrid = new dojox.grid.EnhancedGrid({
			  store: null,
			  structure: layout,
			  autoHeight:true,
			  keepSelection:true, 
			  style: "width:100%; height:95%;margin-left:-2px;padding:0px;",
			  loadingMessage: resources["retrievingUserMessage"],
			  onSelectionChanged: dLang.hitch(this, "_rowSelectionChanged"),
			  selectionMode: 'single'
		  
		  }, document.createElement('div'));
		  this.centerPane.domNode.appendChild(this.userGrid.domNode);
		  this.userGrid.startup();
		  topic.subscribe("userRegistry/refresh", dLang.hitch(this, "topicEvent"));
	},
	
	topicEvent:function() {
		console.log("topic received: ", arguments);
		this._importUserCompleted();
	},
	
	/**
	 * Row selection change handle
	 */
	_rowSelectionChanged:function(event) {
		var selected = this.userGrid.selection.getSelected();
        if (selected.length > 0) {
        	if (this.editEmailButton != null) {
        		this.editEmailButton.set("disabled", false);
        	}
        }
        else {
        	if (this.editEmailButton != null) {
        		this.editEmailButton.set("disabled", true);
        	}
        }
	},
	
	/**
	 * Submit the request to synchronize with LDAP server
	 */
	_synchronizeWithLDAP:function() {
		this.busyDialog = new BusyDialog({message:this.synchonizeMessage, title:resources["synchronizeLDAPTitle"], showCloseButton:false});
		this.busyDialog.startup();
		this.busyDialog.show();
		xhr.post("/optim/rest/security/registry/LDAPSYNCHRONIZE", 
        {
        	handleAs: "text",
        	sync: false,
        }).then(dLang.hitch(this, "_synchronizeComplete"), dLang.hitch(this, "_synchronizeFailed"), function(evt){
        	//do nothing
        	console.log(evt);
        });
	},
	
	/**
	 * Synchronization completed
	 */
	_synchronizeComplete:function(data) {
		if (this.busyDialog != null) {
			this.busyDialog.close();
			this.busyDialog = null;
		}
		this.retrieveUsers();
	},
	
	/**
	 * Synchronized with ldap failed
	 */
	_synchronizeFailed:function(data) {
		console.log(data);
		if (this.busyDialog != null) {
			console.log("busyDialog...");
			this.busyDialog.setErrorMessage(this.synchonizeFailedMessage);
			this.busyDialog.setClosable(true);
		}
	},
	
	/**
	 * Edit email invoked
	 */
	_editEmail:function() {
		var selected = this.userGrid.selection.getSelected();
		var dialog = new EditEmailDialog({selectedUser: selected[0], onComplete:dLang.hitch(this,"retrieveUsers")});
		dialog.startup();
	},
	
	/**
	 * The post startup function
	 */
	postStartup:function() {
		this.inherited(arguments);
	},
	
	/**
	 * Resize this widget
	 */
	resize:function() {
		this.userRegistryBC.resize();
	},
	
	/**
	 * The onShow handler - wait till this widget is display to users to avoid concurrency with the directory connection page
	 */
	onShow:function() {
		if (this.userGrid.store == null) {
			this.retrieveLDAPInfo();
			this.retrieveUsers();
		}
	},
	
	/**
	 * Send the request to retrieve LDAP server information
	 */
	retrieveLDAPInfo:function() {
		xhr.get("/optim/rest/security/registry/ldap", 
	        	{
	        	handleAs: "json"
	        }).then(dLang.hitch(this, "ldapInfoRetrieved"), dLang.hitch(this, "ldapInfoRetrieveFailed"), dLang.hitch(this, "ldapInfoRetrieveFailed"));
	},
	
	/**
	 * LDAP server information retrieval failed - backend is currently reporting a 500 when there's no ldap information stored
	 */
	ldapInfoRetrieveFailed:function(data) {
		console.log(data);
		this.ldapInfoRetrieved(null);
	},
	
	/**
	 * LDAP information retrieved
	 */
	ldapInfoRetrieved:function(data) {
		console.log(data);
		this.ldapInfo = data;
		if (data == "") {
			this.ldapInfo = null;
		}
		if (this.ldapInfo != null) {
			if (this.synchronizeButton == null) {
				this.synchronizeButton = new Button({iconClass:"synchronizeLDAPIcon", label:this.synchronizeLabel});
				this.toolbarNode.addChild(this.synchronizeButton, 1);
				this.synchronizeButton.on("click", dLang.hitch(this, "_synchronizeWithLDAP"));
			}
			if (this.editEmailButton != null) {
				this.toolbarNode.removeChild(this.editEmailButton);
				this.editEmailButton.destroy();
				this.editEmailButton = null;
			}
		}
		else {
			if (this.synchronizeButton != null) {
				this.toolbarNode.removeChild(this.synchronizeButton);
				this.synchronizeButton.destroy();
				this.synchronizeButton = null;
			}
			if (this.editEmailButton == null) {
				console.log("adding email button");
				this.editEmailButton = new Button({iconClass:"editEmailIcon", label:this.editEmailLabel});
				this.toolbarNode.addChild(this.editEmailButton, 1);
				this.editEmailButton.on("click", dLang.hitch(this, "_editEmail"));
				this._rowSelectionChanged(null);
			}
		}
	},
	
	/**
	 * Send the request to users
	 */
	retrieveUsers:function() {
		if (this.userGrid.store != null) {
			this.userGrid.store.close();
		}
		var userStore = new dojo.data.ItemFileWriteStore({
				clearOnClose:true,
				urlPreventCache: true,
				url: "/optim/rest/security/user"
			});
		userStore.fetch({onComplete: dLang.hitch(this,"_userRetrieved")});
		this.userGrid.setStore(userStore);
	},

	/**
	 * Users retrieved
	 */
	_userRetrieved:function(data) {
		console.log("_userRetrieved");
		console.log(data);
		console.log(data.length);
		if (data.length == 0) {
			var message = resources["noUsersFoundMessage"];
			this.userGrid.showMessage("<center><div class='oneuiMsgInfo16' style='padding-left:10px;width:30em;'>" + message + "</div></center>");
			this.resize();
		}
		this._rowSelectionChanged(null);
	},
	
	/**
	 * Import wizard completed
	 */
	_importUserCompleted:function() {
		this.retrieveUsers();
		this.retrieveLDAPInfo();
	},
	
	/**
	 * Display the import wizard
	 */
	importUsers:function() {
		var dialog = new admin.users.dialogs.ImportUserWizardDialog({"ldapInfo": this.ldapInfo});
		//dialog.on("finish", dLang.hitch(this, "_importUserCompleted"));
		dialog.startup();
	}
});

});